require 'minitest/autorun'

require 'process_metrics'

class MyClass
  include ProcessMetrics::Timer
  measure :do_simple_work,
          :do_work_with_params,
          :do_work_with_block,
          :do_work_with_params_and_block,
          :do_terrible_work

  attr_accessor :my_var

  def initialize(my_var)
    @my_var = my_var
  end

  def do_simple_work
    @my_var + 1
  end

  def do_work_with_block(&block)
    block.call(self)
  end

  def do_work_with_params(user_id)
    @my_var + user_id + 3
  end

  def do_work_with_params_and_block(user_id, &block)
    block.call self
    @my_var + user_id + 5
  end

  def do_terrible_work
    raise "oh no!"
  end
end


class MyChildSymbolClass
  include ProcessMetrics::Timer
  measure :do_simple_work,
          parent_uuid: :parent_object_uuid

  attr_accessor :my_var

  def initialize(my_var)
    @my_var = my_var
  end

  def do_simple_work
    @my_var + 1
  end

  def parent_object_uuid
    "fe51cfe1-114f-4516-8c5f-e488e48c3778"
  end
end

class MyChildProcClass
  include ProcessMetrics::Timer
  measure :do_simple_work,
          parent_uuid: -> obj { "b0d50400-cae0-4cc4-b289-f487d3c9b397" }

  attr_accessor :my_var

  def initialize(my_var)
    @my_var = my_var
  end

  def do_simple_work
    @my_var + 1
  end
end

class MyChildStringClass
  include ProcessMetrics::Timer
  measure :do_simple_work,
          parent_uuid: "c9b104fb-3389-4c21-b0e9-6c88fd905e4f"

  attr_accessor :my_var

  def initialize(my_var)
    @my_var = my_var
  end

  def do_simple_work
    @my_var + 1
  end
end

class TestBlock < Minitest::Test
  def test_unfinished_job
    my_object = MyClass.new(1)
    assert_raises(RuntimeError) { my_object.do_terrible_work }
  end

  def test_simple_method
    my_object = MyClass.new(1)

    assert_equal 2, my_object.do_simple_work
  end

  def test_method_with_params
    my_object = MyClass.new(2)

    assert_equal 8, my_object.do_work_with_params(3)
  end

  def test_method_with_block
    my_object = MyClass.new(5)

    my_object.do_work_with_block do |that_object|
      that_object.my_var += 7
    end

    assert_equal 12, my_object.my_var
  end

  def test_method_with_params_and_block
    my_object = MyClass.new(11)

    result = my_object.do_work_with_params_and_block(13) do |that_object|
      that_object.my_var += 17
    end

    assert_equal 46, result
  end

  def test_child_symbol_simple_method
    my_object = MyChildSymbolClass.new(1)

    assert_equal 2, my_object.do_simple_work
  end

  def test_child_proc_simple_method
    my_object = MyChildProcClass.new(1)

    assert_equal 2, my_object.do_simple_work
  end

  def test_child_string_simple_method
    my_object = MyChildStringClass.new(1)

    assert_equal 2, my_object.do_simple_work
  end

  def test_block
    ProcessMetrics.measure('anon') do |metric|
      sleep rand
      metric.data = { my_key: 'my_value'}
    end
  end
end
